@@ -176,7 +176,7 @@ anjuta_shell_thaw (AnjutaShell *shell, GError **error)
 	if (freeze_count <= 0)
 	{
 		/* Add all pending widgets */
-		/* DEBUG_PRINT ("Thawing shell ..."); */
+		/* DEBUG_PRINT ("%s", "Thawing shell ..."); */
 		
 		GQueue *queue;
 		queue = g_object_get_data (G_OBJECT (shell), "__widget_queue");
@@ -754,6 +754,34 @@ anjuta_shell_get_profile_manager (AnjutaShell *shell, GError **error)
 	return ANJUTA_SHELL_GET_IFACE (shell)->get_profile_manager (shell, error);
 }
 
+/**
+ * anjuta_shell_saving_push:
+ * @shell: A #AnjutaShell interface
+ *
+ * Increase the count of files that need to be saved 
+ *
+ */
+void anjuta_shell_saving_push	    (AnjutaShell* shell)
+{
+	g_return_if_fail (ANJUTA_IS_SHELL (shell));
+	
+	return ANJUTA_SHELL_GET_IFACE (shell)->saving_push (shell);
+}
+
+/**
+ * anjuta_shell_saving_push:
+ * @shell: A #AnjutaShell interface
+ *
+ * Decrease the count of files that need to be saved 
+ *
+ */
+void anjuta_shell_saving_pop	    (AnjutaShell* shell)
+{
+	g_return_if_fail (ANJUTA_IS_SHELL (shell));
+
+	return ANJUTA_SHELL_GET_IFACE (shell)->saving_pop (shell);
+}
+
 void
 anjuta_shell_session_save (AnjutaShell *shell, const gchar *session_directory,
 						   GError **error)
@@ -787,7 +815,7 @@ anjuta_shell_session_load (AnjutaShell *shell, const gchar *session_directory,
 	/* Do not allow multiple session loadings at once. Could be a trouble */
 	if (g_object_get_data (G_OBJECT (shell), "__session_loading"))
 	{
-		/* DEBUG_PRINT ("A session load is requested in the middle of another!!"); */
+		/* DEBUG_PRINT ("%s", "A session load is requested in the middle of another!!"); */
 		return;
 	}
 	g_object_set_data (G_OBJECT (shell), "__session_loading", "1");
