@@ -187,6 +187,12 @@ symbol_db_view_locals_clear_cache (SymbolDBViewLocals *dbvl)
 	
 	priv = dbvl->priv;
 		
+	if (priv->insertion_idle_handler > 0)
+	{				
+		g_source_remove (priv->insertion_idle_handler);
+		priv->insertion_idle_handler = 0;
+	}
+	
 	/*DEBUG_PRINT ("%s", "symbol_db_view_locals_clear_cache ()");*/
 	/* check whether we already have the view status saved on hash table or not.
 	 * If we saved that then don't remove it, or there may be a memory leak
@@ -1371,7 +1377,7 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 	priv->current_local_file_path = NULL;
 
 	priv->current_db_file = 
-		symbol_db_engine_get_file_db_path (dbe, filepath);
+		symbol_db_util_get_file_db_path (dbe, filepath);
 	if (priv->current_db_file == NULL) 
 	{
 		DEBUG_PRINT ("Warning: priv->current_db_file is NULL");
